/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Tinderbox Status Summary.
 *
 * The Initial Developer of the Original Code is
 * Mark Banner <bugzilla@standard8.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

if (!Cc)
  var Cc = Components.classes;
if (!Ci)
  var Ci = Components.interfaces;

var gPrefService = Cc["@mozilla.org/preferences-service;1"]
                   .getService(Ci.nsIPrefService);
var gPrefBranch = gPrefService.getBranch("extensions.tinderstatus.");

const kNumWatchLists = 10;

function onLoadTinderstatusPrefsPane()
{
  var gotoTreePopup = document.getElementById("gotoTreePopup");
  var fullList = gPrefBranch.getComplexValue("fullList",
                                             Ci.nsISupportsString).data;
  var gotoTreePref = gPrefBranch.getComplexValue("gotoTree",
                                                 Ci.nsISupportsString).data;
  fullList = fullList.split(",");

  var i;
  var watchList = [ { element: document.getElementById("watchList"),
		      pref: gPrefBranch.getComplexValue("watchList",
							Ci.nsISupportsString).data.split(",")
                      } ];

  for (i = 1; i < kNumWatchLists; ++i) {
    var item = { element: document.getElementById("watchList" + i) };
    try {
      var prefString = gPrefBranch.getComplexValue("watchList" + i,
						   Ci.nsISupportsString).data;
      item.pref = prefString.split(",");
    }
    catch (ex) {
      item.pref = [];
    }
    watchList.push(item);
  }

  watchList.push({ element: document.getElementById("notifyTrees"),
                   pref: gPrefBranch.getComplexValue("notifyTrees",
                                                     Ci.nsISupportsString).data.split(",") });
  watchList.push({ element: document.getElementById("notifyTinderboxes"),
                   pref: gPrefBranch.getComplexValue("notifyTinderboxes",
                                                     Ci.nsISupportsString).data.split(",") });

  var item;

  for (i = 0; i < fullList.length; ++i) {
    // Add the item to the watch lists
    for (var j = 0; j < watchList.length; ++j) {
      item = document.createElement("listitem");
      item.setAttribute("id", fullList[i]);
      item.setAttribute("label", fullList[i]);
      item.setAttribute("type", "checkbox");
      item.setAttribute("checked", watchList[j].pref.indexOf(fullList[i]) != -1);
      watchList[j].element.appendChild(item);
    }

    // Add the item to the goto tree
    item = document.createElement("menuitem");
    item.setAttribute("label", fullList[i]);
    item.setAttribute("value", fullList[i]);
    gotoTreePopup.appendChild(item);

    if (fullList[i] == gotoTreePref) {
      gotoTreePopup.parentNode.value = fullList[i];
      gotoTreePopup.selectedItem = item;
    }
  }
}

function syncToList(aElement)
{
  var wlArray = [];
  for (var i = 0; i < aElement.getRowCount(); ++i) {
    var wlNode = aElement.getItemAtIndex(i);
    if (wlNode.checked)
      wlArray.push(wlNode.id);
  }
  return wlArray.join(",");
}
