/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Tinderbox Status Summary.
 *
 * The Initial Developer of the Original Code is
 * Myk Melez <myk@melez.com>.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Alexander J. Vincent <ajvincent@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// This file was completely rewritten in June, 2005 by Alexander J. Vincent.

const tsOptionsURL = "chrome://tinderstatus/content/prefs.xul";

if (!Cc)
  var Cc = Components.classes;
if (!Ci)
  var Ci = Components.interfaces;

// FIXME: don't set a bunch of non-namespaced global variables.

var gXMLHttpRequest;

// The order in this list somewhat matters, as the first item is the one
// that gets loaded when you click on the statusbar icon.
var gTinderboxIndex = -1;
var gTinderboxText  = "";
var gTinderboxURIIndex = 0;
var gTinderboxIntervalID = 0;
var gTinderboxLastUpdate = null;

// FIXME: move all the top-level variables into this object so they don't
// pollute the global namespace and potentially conflict with other extensions
// and core code.
var Tinderstatus = {
  _numTrees: 1,
  _treeBoxes: null,
  _internalTinderboxQueryList: [],

  // An array of the current tinderbox list and the specific results for the
  tinderboxDetails: [],

  // This flag is used so that we don't update the tinderboxDetails whilst
  // we're in the middle of a set of queries
  _tinderboxListUpdateRequired: true,

  init: function() {
    // If we haven't yet converted references to the Firefox3.1 tree into
    // references to the Firefox3.5 tree (to which it was renamed), do so now.
    // FIXME: remove this code after everyone has updated to Tinderstatus 0.3.1
    // or a newer version.
    if (!this._prefs.getBoolPref("renamed_Firefox3.1_to_Firefox3.5")) {
      var i = 0, watchList = "watchList";
      // Go through watchlist prefs and update the ones with user set values.
      while (this._prefs.getPrefType(watchList) != Ci.nsIPrefBranch.PREF_INVALID) {
        if (this._prefs.prefHasUserValue(watchList)) {
          var oldWatchList = this._prefs.getComplexValue(watchList, Ci.nsISupportsString).data;
          var newWatchList = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
          newWatchList.data =
            oldWatchList.split(",").                                              // split into array of trees
                         map(function(v) v == "Firefox3.1" ? "Firefox3.5" : v).   // Firefox3.1 -> Firefox3.5
                         sort().                                                  // sort to enable duplicate filtration
                         filter(function(v, i, a) i == 0 || v != a[i-1]).         // remove duplicates (i.e. Firefox3.5 twice)
                         join(",");                                               // join into string of trees
          this._prefs.setComplexValue(watchList, Ci.nsISupportsString, newWatchList);
        }
        watchList = "watchList" + ++i;
      }
      this._prefs.setBoolPref("renamed_Firefox3.1_to_Firefox3.5", true);
    }

    this._numTrees = this._prefs.getIntPref("numTrees");
    // Must have at least one tree.
    if (this._numTrees <= 0)
      this._numTrees = 1;

    let prefValue = "";
    try {
      prefValue = Tinderstatus._prefs.getComplexValue("watchList", Ci.nsISupportsString).data;
    }
    catch (e) { }

    let toolTip = document.getElementById("tinderstatus-tooltip");

    this._treeBoxes = [ { box: document.getElementById("tinderstatus-box"),
                          tooltip: toolTip,
                          trees: prefValue } ];
  },

  updateCurrentTinderboxList: function () {
    let oldTinderboxDetails = this.tinderboxDetails;
    this.tinderboxDetails = [];
    function addDetails(element) {
      let oldDetails = null;
      for (let i = 0; i < oldTinderboxDetails.length; ++i) {
        if (oldTinderboxDetails[i].tree == element)
          oldDetails = oldTinderboxDetails[i];
      }
      Tinderstatus.tinderboxDetails.push(oldDetails ? oldDetails :
                                         {tree: element,
                                          result: "",
                                          lastStatus: "Unknown" });
    }

    this._internalTinderboxQueryList.forEach(addDetails);
    this._tinderboxListUpdateRequired = false;
  },

  // This function just goes through everything (number of trees/what to watch)
  // and refreshes everything.
  updateTreeIconsAndPrefs: function() {
    var i;

    this._numTrees = this._prefs.getIntPref("numTrees");
    // Must have at least one tree.
    if (this._numTrees <= 0)
      this._numTrees = 1;

    let firstBox = Tinderstatus._treeBoxes[0].box;
    let firstTip = Tinderstatus._treeBoxes[0].tooltip;
    let parentBox = firstBox.parentNode;
    let parentTip = firstTip.parentNode;

    // If the user has reduced the number of boxes
    for (i = this._treeBoxes.length - 1; i >= this._numTrees; --i) {
      parentBox.removeChild(this._treeBoxes[i].box);
      parentTip.removeChild(this._treeBoxes[i].tooltip);
      this._treeBoxes.pop();
    }

    // If the user has increased the number of boxes
    for (i = this._treeBoxes.length; i < this._numTrees; ++i) {
      let newBox = firstBox.cloneNode(true);
      newBox.id = "tinderstatus-box-" + i;
      newBox.setAttribute("buildstatus", "none");
      newBox.firstChild.setAttribute("treestatus", "none");

      let newTip = firstTip.cloneNode(true);
      newTip.id = "tinderstatus-tooltip-" + i;
      newTip.firstChild.setAttribute("value", "No status retrieved yet.");

      newBox.setAttribute("tooltip", newTip.id);

      // trees gets filled in below.
      this._treeBoxes[i] = { box: newBox, tooltip: newTip, trees: "" };

      parentBox.insertBefore(newBox, Tinderstatus._treeBoxes[i - 1].box.nextSibling);
      parentTip.insertBefore(newTip, Tinderstatus._treeBoxes[i - 1].tooltip.nextSibling);
    }

    // Clear out the list of tinderboxes to query.
    var newList = [];

    // Now update the prefs for all the tree boxes
    for (i = 0; i < this._treeBoxes.length; ++i) {
      var prefValue = "";
      try {
	prefValue =
          this._prefs.getComplexValue(i > 0 ? "watchList" + i : "watchList",
                                      Ci.nsISupportsString).data;
      }
      catch (e) { }

      this._treeBoxes[i].trees = prefValue;
      this._treeBoxes[i].tooltip
          .childNodes[1].setAttribute("value", "Watching: " + prefValue);
      newList = newList.concat(prefValue.split(","));
    }

    // Also add any we're getting notifications for, but not actually watching
    var notifyTrees = this._prefs.getCharPref("notifyTinderboxes") + "," +
                      this._prefs.getCharPref("notifyTrees");

    newList = newList.concat(notifyTrees.split(","));

    // Now strip out any duplicates
    this._internalTinderboxQueryList = [];
    for (i = 0; i < newList.length; ++i) {
      if (newList[i] &&
          this._internalTinderboxQueryList.indexOf(newList[i], 0) < 0)
	this._internalTinderboxQueryList.push(newList[i]);
    }
    this._tinderboxListUpdateRequired = true;
  },

  translateStatus: function (text) {
    if (!text)
      return "Unknown";

    let status = "Unknown";

    if (text.indexOf("busted") != -1) {
      status = "Busted";
    }
    else if (text.indexOf("testfailed") != -1) {
      status = "Test Failed";
    }
    else if (text.indexOf("exception") != -1) {
      status = "Non-build failure";
    }
    else if (text.indexOf("success") != -1) {
      status = "Success";
    }

    return status;
  },

  checkUpdatedTree: function (index) {
    let currentStatus = this.translateStatus(this.tinderboxDetails[index].result);
    let notifyTrees = this._prefs.getCharPref("notifyTrees").split(",");
    let treeName = this.tinderboxDetails[index].tree;
    let lastStatus = this.tinderboxDetails[index].lastStatus;

    if (notifyTrees.indexOf(treeName) != -1 &&
        lastStatus != currentStatus &&
        currentStatus != "Unknown" && lastStatus != "Unknown") {
      this.showAlert(treeName, "Tree state is now: " + currentStatus);
    }
    this.tinderboxDetails[index].lastStatus = currentStatus;
  },

  checkUpdatedTinderboxes: function (index) {
    let currentText = this.tinderboxDetails[index].result;
    let boxes = currentText.split("\n");

    let notifyTinderboxes = this._prefs.getCharPref("notifyTinderboxes").split(",");

    let treeName = this.tinderboxDetails[index].tree;

    if (notifyTinderboxes.indexOf(treeName) == -1)
      return;

    for (let i = 0; i < boxes.length; ++i) {
      let details = boxes[i].split("|");
      let boxName = details[2];
      let boxStatus = this.translateStatus(details[3]);

      if (!(boxName in this.tinderboxDetails[index])) {
        this.tinderboxDetails[index][boxName] = {
          status: boxStatus,
          time: details[4]
        };
        // New box, no point in progressing here.
        continue;
      }

      if (boxStatus != "Unknown") {
        let lastBoxStatus = this.tinderboxDetails[index][boxName].status;
        let lastBoxTime = this.tinderboxDetails[index][boxName].time;

        // Show an alert if the status has changed
        if (lastBoxStatus != boxStatus && lastBoxStatus != "Unknown")
          this.showAlert(treeName, "Builder: " + boxName + " Status: " + boxStatus);
        // Show an alert if the status isn't success and a new build has
        // completed but not changed state.
        else if (lastBoxStatus == boxStatus && boxStatus != "Success" &&
            lastBoxTime != details[4])
          this.showAlert(treeName, "Builder: " + boxName + " is still " + boxStatus);
      }

      this.tinderboxDetails[index][boxName].status = boxStatus;
      this.tinderboxDetails[index][boxName].time = details[4];
    }
  },

  get _prefSvc() {
    delete this._prefSvc;
    return this._prefSvc = Cc["@mozilla.org/preferences-service;1"].
                           getService(Ci.nsIPrefService);
  },

  // FIXME: use the Preferences module to make this easier.
  // https://wiki.mozilla.org/Labs/JS_Modules#Preferences
  get _prefs() {
    delete this._prefs;
    return this._prefs = this._prefSvc.
                         getBranch("extensions.tinderstatus.").
                         QueryInterface(Ci.nsIPrefBranch2);
  },

  onContextShowing: function () {
    document.getElementById("tinderstatus-notifications").
      setAttribute("checked", this._prefs.getBoolPref("notifications"));
  },

  onNotificationToggled: function () {
    this._prefSvc.QueryInterface(Ci.nsIPrefBranch).setBoolPref("extensions.tinderstatus.notifications", !!document.getElementById("tinderstatus-notifications").getAttribute("checked"));
  },

  showAlert: function (title, text) {
    if (this._prefs.getBoolPref("notifications")) {
      let alertsService = Components.classes["@mozilla.org/alerts-service;1"]
        .getService(Components.interfaces.nsIAlertsService);

      alertsService.showAlertNotification("Tree State", title, text);
    }
  }
};

var tinderstatusPrefObserver =
{
  register: function()
  {
    Tinderstatus._prefs.addObserver("", this, false);
  },

  unregister: function()
  {
    Tinderstatus._prefs.removeObserver("", this);
  },

  observe: function(aSubject, aTopic, aData)
  {
    if (aTopic != "nsPref:changed")
      return;

    if (aData.lastIndexOf("watchList", 0) != -1 ||
	aData == "numTrees") {

      Tinderstatus.updateTreeIconsAndPrefs();

      refreshTinderboxenStatus();
      return;
    }

    switch (aData) {
      case "refreshTime":
        if (gTinderboxIntervalID)
          window.clearInterval(gTinderboxIntervalID);

        gTinderboxIntervalID = window.setInterval(refreshTinderboxenStatus,
                                                  getRefreshTime());
        break;
      case "fullList":
        setUpTinderboxMenuPopup();
        break;
    }
  }
};

function initializeTinderstatus() {
  Tinderstatus.init();

  var appInfo = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo);

  // If we're running in SeaMonkey and the version is less than 2.*, then the
  // options dialog won't work, so disable the menu option.
  if (appInfo.ID == "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}") {
    var versionChecker = Cc["@mozilla.org/xpcom/version-comparator;1"]
      .getService(Ci.nsIVersionComparator);

    if (versionChecker.compare(appInfo.version, "2.0a1pre") < 0)
      document.getElementById("tinderstatus-prefs").hidden = true;
  }

  // If we're not on windows, use preferences for the menu instead of Options.
  if (navigator.platform.toLowerCase().indexOf('win') == -1)
    document.getElementById("tinderstatus-prefs").label = "Preferences";

  // Set up the tree icons and prefs
  Tinderstatus.updateTreeIconsAndPrefs();

  // Kick off the first update.
  refreshTinderboxenStatus();

  // Setup the menu
  setUpTinderboxMenuPopup();
}

function setUpTinderboxMenuPopup() {
  // Get the popup and remove any existing items from it.
  var popup = document.getElementById("tinderstatus-context");
  var separatorItem = document.getElementById("tinderstatus-separator");
  var openTreeItem = document.getElementById("tinderstatus-open-tree");

  var nextitem = separatorItem.previousSibling;
  while (nextitem != openTreeItem) {
    popup.removeChild(nextitem);
    nextitem = separatorItem.previousSibling;
  }

  // Get the full list of tinderboxes that we know about.
  var fullList;

  try {
    fullList = Tinderstatus._prefs.getComplexValue("fullList", Ci.nsISupportsString).data;
  }
  catch (ex) {
    fullList = "Bugzilla,Camino,Firefox,Firefox3.0,Firefox3.5,Firefox-Ports,Mobile,SeaMonkey,SeaMonkey-Ports,Sunbird,Thunderbird,Thunderbird3.0,XULRunner";
  }

  fullList = fullList.split(",");

  // Now add the tinderboxes onto the menu
  var item;
  for (var i = 0; i < fullList.length; ++i) {
    item = document.createElement("menuitem");
    item.setAttribute("label", fullList[i]);
    item.setAttribute("oncommand", "loadTinderboxPage('" + fullList[i] + "');");
    popup.insertBefore(item, separatorItem);
  }
}

function openTinderstatusPreferences() {
  var windows = Cc['@mozilla.org/appshell/window-mediator;1']
    .getService(Ci.nsIWindowMediator)
    .getEnumerator(null);

  while (windows.hasMoreElements()) {
    var win = windows.getNext();
    if (win.document.documentURI == tsOptionsURL) {
      win.focus();
      return;
    }
  }

  var features;
  try {
    var instantApply = Tinderstatus._prefSvc.getBranch(null).
                       getBoolPref("browser.preferences.instantApply");
    features = "chrome,titlebar,toolbar,centerscreen" + (instantApply ? ",dialog=no" : ",modal");
  }
  catch (e) {
    features = "chrome,titlebar,toolbar,centerscreen,modal";
  }
  openDialog(tsOptionsURL, "", features);
}

function getRefreshTime() {
  var refreshTime;

  try {
    // refreshTime is stored in minutes
    refreshTime = Tinderstatus._prefs.getIntPref("refreshTime") * 60000;

    // Always ensure refresh time is at least one minute so we don't hit
    // the servers hard
    if (refreshTime < 60000)
      refreshTime = 60000;
  }
  catch(ex) {
    // set to default of one minute.
    refreshTime = 60000;
  }

  return refreshTime;
}

/**
 * Load into the browser a current tinderbox page.
 */
function loadTinderboxURI(evt) {
  // Only load for left or middle click events.
  if (evt.button > 1)
    return;

  var tinderboxToLoad;
  try {
    tinderboxToLoad = Tinderstatus._prefs.getComplexValue("gotoTree", Ci.nsISupportsString).data;
  }
  catch (ex) {
    // If we can't get the pref, set to the default
    tinderboxToLoad = "Firefox";
  }

  if (tinderboxToLoad != "") {
    var url = "http://tinderbox.mozilla.org/" + tinderboxToLoad + "/";
    if (!handleLinkClick(evt, url, null))
      loadURI(url);
  }
}

function loadTinderboxPage(pageName) {
  loadURI("http://tinderbox.mozilla.org/" + pageName + "/");
}

function refreshTinderboxenStatus() {
  // Check we aren't currently doing an update
  if (gTinderboxIndex != -1)
    return;

  Tinderstatus.updateCurrentTinderboxList();

  gTinderboxURIIndex = 0;
  getTinderboxStatus();
}

/**
 * Request a tinderbox summary from one tree.
 */
function getTinderboxStatus() {
  // If no items selected, just show nothing.
  if (Tinderstatus.tinderboxDetails.length == 0) {
    document.getElementById('tinderstatus-box')
            .setAttribute("buildstatus", "none");

    //    setTinderboxStatus("No tinderboxes selected.");
    gTinderboxURIIndex = 0;
    return;
  }

  gTinderboxIndex++;
  if (gTinderboxIndex == Tinderstatus.tinderboxDetails.length) {
    return;
  }
  var url = "http://tinderbox.mozilla.org/";
  url += Tinderstatus.tinderboxDetails[gTinderboxIndex].tree;
  url += "/quickparse.txt";

  // Oh, how I wish we didn't have to reinitialize every time.
  gXMLHttpRequest = new XMLHttpRequest();
  gXMLHttpRequest.onload = displayTinderboxStatus;
  gXMLHttpRequest.onerror = handleRequestError;
  gXMLHttpRequest.open("GET", url);
  gXMLHttpRequest.send(null);
}

function handleRequestError() {
  // Empty out the result for the current request.
  Tinderstatus.tinderboxDetails[gTinderboxIndex].result = "";
  // Abort current attempt, wait for next refresh
  gTinderboxIndex = -1;
  gTinderboxText = "";
}

function getSetLastUpdated() {
  gTinderboxLastUpdate = new Date();

  var result = "Last Updated: ";

  if (gTinderboxLastUpdate.getHours() < 10)
    result += "0";

  result += gTinderboxLastUpdate.getHours();

  result += gTinderboxLastUpdate.getMinutes() < 10 ? ":0" : ":";

  result += gTinderboxLastUpdate.getMinutes();

  result += gTinderboxLastUpdate.getSeconds() < 10 ? ":0" : ":";

  return result + gTinderboxLastUpdate.getSeconds();
}

/**
 * Indicate via tooltip tinderboxes which show failures.
 */
function fillTinderboxStatus(aTooltip, aTinderboxText, aRegExp) {
  var results = aTinderboxText.match(aRegExp);

  if (aTooltip.firstChild == aTooltip.lastChild)
    throw "Error, should be two elements in tooltip not one.";

  while (aTooltip.childNodes[1] != aTooltip.lastChild) {
    aTooltip.removeChild(aTooltip.lastChild);
  }

  aTooltip.firstChild.setAttribute("value", getSetLastUpdated());

  for (var i = 0; i < results.length; i++) {
    var node = document.createElement("description");
    node.setAttribute("value", results[i]);
    aTooltip.appendChild(node);
  }
}

/**
 * Indicate via tooltip a status message (usually success).
 */
function setTinderboxStatus(aTooltip, aMessage) {
  if (aTooltip.firstChild == aTooltip.lastChild)
    throw "Error, should be two elements in tooltip not one.";

  while (aTooltip.childNodes[1] != aTooltip.lastChild) {
    aTooltip.removeChild(aTooltip.lastChild);
  }

  aTooltip.firstChild.setAttribute("value", getSetLastUpdated());

  var node = document.createElement("description");
  node.setAttribute("value", aMessage);
  aTooltip.appendChild(node);
}

/**
 * Handle response from tinderbox server, and update status bar.
 */
function displayTinderboxStatus()
{
  var text = gXMLHttpRequest.responseText + "\n";
  gTinderboxText += text;
  Tinderstatus.tinderboxDetails[gTinderboxIndex].result = text;

  Tinderstatus.checkUpdatedTree(gTinderboxIndex);
  Tinderstatus.checkUpdatedTinderboxes(gTinderboxIndex);

  // Note which tinderbox fails.
  if ((gTinderboxURIIndex == 0) &&
      ((text.indexOf("busted") != -1) || (text.indexOf("testfailed") != -1) ||
       (text.indexOf("exception") != -1))) {
    gTinderboxURIIndex = gTinderboxIndex;
  }

  // Do we need to loop around again?
  if (gTinderboxIndex < (Tinderstatus.tinderboxDetails.length - 1)) {
    getTinderboxStatus();
    return;
  }

  // Change the background color of the box depending to reflect build status.
  for (let i = 0; i < Tinderstatus._treeBoxes.length; ++i) {
    let box = Tinderstatus._treeBoxes[i].box;
    let tooltip = Tinderstatus._treeBoxes[i].tooltip;
    let prefs = Tinderstatus._treeBoxes[i].trees.split(",");
    let reducedText = "";
    for (let j = 0; j < prefs.length; ++j) {
      for (let k = 0; k < Tinderstatus.tinderboxDetails.length; ++k)
        if (Tinderstatus.tinderboxDetails[k].tree == prefs[j])
          reducedText += Tinderstatus.tinderboxDetails[k].result + "\n";
    }

    let re;
    if (reducedText.indexOf("busted") != -1) {
      re = /.*(busted|testfailed|exception)/g;
      fillTinderboxStatus(tooltip, reducedText, re);
      box.setAttribute("buildstatus", "busted");
    }
    else if (reducedText.indexOf("testfailed") != -1) {
      re = /.*(testfailed|exception)/g;
      fillTinderboxStatus(tooltip, reducedText, re);
      box.setAttribute("buildstatus", "testfailed");
    }
    else if (reducedText.indexOf("exception") != -1) {
      re = /.*exception/g;
      fillTinderboxStatus(tooltip, reducedText, re);
      box.setAttribute("buildstatus", "exception");
    }
    else if (reducedText.indexOf("success") != -1) {
      setTinderboxStatus(tooltip, "All tinderboxes are green.");
      box.setAttribute("buildstatus", "success");
      gTinderboxURIIndex = 0;
    }
    else {
      setTinderboxStatus(tooltip, "Unknown tinderbox status.");
      box.setAttribute("buildstatus", "none");
      gTinderboxURIIndex = 0;
    }

    // Change the icon superimposed on the box to reflect tree status.
    var icon = box.firstChild;
    if (reducedText.indexOf("closed") != -1)
      icon.setAttribute("treestatus", "closed");
    else if (reducedText.indexOf("open") != -1)
      icon.setAttribute("treestatus", "open");
    else
      icon.setAttribute("treestatus", "none");
  }

  gTinderboxIndex = -1;
  gTinderboxText = "";

  // If we have just done an update, and the an update request is signalled
  // due to change of pref, run the update again
  if (Tinderstatus._tinderboxListUpdateRequired)
    refreshTinderboxenStatus();
}

tinderstatusPrefObserver.register();

// Check the status a second after the browser loads.
window.addEventListener("load", function initTinderboxStatus(evt) {
  window.setTimeout(initializeTinderstatus, 1000);

  // Wait one minute before trying again.
  gTinderboxIntervalID = window.setInterval(refreshTinderboxenStatus,
                                            getRefreshTime());
}, false);
